@@ -57,6 +57,21 @@
 #include <unistd.h> 
 #include <dirent.h>
 #include <errno.h>
+#include <sys/types.h>
+#ifdef HAVE_SYS_ACL_H
+#include <sys/acl.h>
+#endif /* HAVE_SYS_ACL_H */
+#if HAVE_FTS_H
+#include <fts.h>
+#endif /* HAVE_FTS_H */
+
+#if !HAVE_DECL_SETENV
+extern int setenv (const char *name, const char *value, int replace);
+#endif
+#if !HAVE_DECL_UNSETENV
+extern int unsetenv (const char *name);
+#endif
+
 
 /* 
    Where are those shared libraries? 
@@ -137,7 +152,7 @@ void load_library_symbols(void){
 
   static int done=0;
   int i;
-  char* msg;
+  const char *msg;
   
   if(!done){
     for(i=0; next_wrap[i].doit; i++){
@@ -650,9 +665,6 @@ int WRAP_FSTATAT64 FSTATAT64_ARG(int ver,
 
 #endif /* STAT64_SUPPORT */
 
-
-
-
 /*************************************************************/
 /*
   Wrapped functions general info:
@@ -844,6 +856,10 @@ int chmod(const char *path, mode_t mode){
   r=next_chmod(path, mode);
   if(r&&(errno==EPERM))
     r=0;
+#ifdef EFTYPE		/* available under FreeBSD kernel */
+  if(r&&(errno==EFTYPE))
+    r=0;
+#endif
   return r;
 }
 
@@ -872,6 +888,10 @@ int fchmod(int fd, mode_t mode){
   r=next_fchmod(fd, mode);
   if(r&&(errno==EPERM))
     r=0;
+#ifdef EFTYPE		/* available under FreeBSD kernel */
+  if(r&&(errno==EFTYPE))
+    r=0;
+#endif
   return r;
 }
 
@@ -904,6 +924,10 @@ int fchmodat(int dir_fd, const char *path, mode_t mode, int flags) {
   r=next_fchmodat(dir_fd, path, mode, flags);
   if(r&&(errno==EPERM))
     r=0;
+#ifdef EFTYPE		/* available under FreeBSD kernel */
+  if(r&&(errno==EFTYPE))
+    r=0;
+#endif
   return r;
 }
 #endif /* HAVE_FCHMODAT */
@@ -1453,3 +1477,28 @@ int fakeroot_isdisabled(void)
 {
   return fakeroot_disabled;
 }
+
+#ifdef HAVE_SYS_ACL_H
+int acl_set_fd(int fd, acl_t acl) {
+  errno = ENOTSUP;
+  return -1;
+}
+
+int acl_set_file(const char *path_p, acl_type_t type, acl_t acl) {
+  errno = ENOTSUP;
+  return -1;
+}
+#endif /* HAVE_SYS_ACL_H */
+
+#ifdef HAVE_FTS_READ
+FTSENT *fts_read(FTS *ftsp) {
+  FTSENT *r;
+
+  r=next_fts_read(ftsp);
+  if(r) {
+    send_get_stat64(r->fts_statp);
+  }
+
+  return r;
+}
+#endif /* HAVE_FTS_READ */
